/*
 * fruconfig.c
 *
 * This tool reads the FRU configuration, and optionally sets the asset
 * tag field in the FRU data.  See IPMI v1.5 spec section 28.
 * It can use either the Intel /dev/imb driver or VALinux /dev/ipmikcs.
 *
 * Author:  Andy Cress  arcress@users.sourceforge.net
 *
 * Copyright (c) 2001-2002 Intel Corporation. 
 *
 * 10/28/02 Andy Cress - created
 * 12/11/02 Andy Cress v0.9  - disable write until checksum fixed.
 * 12/13/02 Andy Cress v0.91 - don't abort if cc=c9 in load_fru loop.
 * 01/27/03 Andy Cress v0.92 - more debug, do checksums,
 * 01/28/03 Andy Cress v1.0  do writes in small chunks, tested w SCB2 & STL2
 * 02/19/03 Andy Cress v1.1  also get System GUID
 * 03/10/03 Andy Cress v1.2  do better bounds checking on FRU size
 * 04/30/03 Andy Cress v1.3  Board Part# & Serial# reversed
 * 05/13/03 Andy Cress v1.4  Added Chassis fields
 * 06/19/03 Andy Cress v1.5  added errno.h (email from Travers Carter)
 */
/*M*
Copyright (c) 2002, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

  a.. Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer. 
  b.. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution. 
  c.. Neither the name of Intel Corporation nor the names of its contributors 
      may be used to endorse or promote products derived from this software 
      without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *M*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <syslog.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "ipmicmd.h" 

extern int errno;

char progname[] = "fruconfig";
char progver[] = "1.5";
char fdebug = 0;
char fasset = 0;
uchar guid[17] = "";
uchar *frubuf = NULL;
int    sfru = 0;
int    asset_offset = -1;
int    asset_len = 0;
char  *asset_tag;
int    maxprod = 0;
int ctype;

#define STRING_DATA_TYPE_BINARY         0x00
#define STRING_DATA_TYPE_BCD_PLUS       0x01
#define STRING_DATA_TYPE_SIX_BIT_ASCII  0x02
#define STRING_DATA_TYPE_LANG_DEPENDANT 0x03

#define NUM_BOARD_FIELDS    6
#define NUM_PRODUCT_FIELDS  8
#define NUM_CHASSIS_FIELDS  3
#define MAX_CTYPE   26
char *ctypes[MAX_CTYPE] = { "", "Other", "Unknown", "Desktop", 
	"Low Profile Desktop", "Pizza Box", "Mini-Tower", "Tower", 
	"Portable", "Laptop", "Notebook", "Handheld", "Docking Station", 
	"All-in-One", "Sub-Notebook", "Space-saving", "Lunch Box", 
	"Main Server Chassis", "Expansion Chassis", "SubChassis", 
	"Bus Expansion Chassis", "Peripheral Chassis", 
	"RAID Chassis", /*0x17=23.*/ "Rack-Mount Chassis", 
	"New24" , "New25"};
	/* what about bladed chassies? */
char *ctype_hdr = 
"Chassis Type        :"; 
char *chassis[NUM_CHASSIS_FIELDS] = {
"Chassis Part Number :", 
"Chassis Serial Num  :",
"Chassis OEM Field   :" };
char *board[NUM_BOARD_FIELDS] = {
"Board Manufacturer  :",
"Board Product Name  :",
"Board Serial Number :",
"Board Part Number   :",
"Board FRU File ID   :",
"Board OEM Field     :" };
char *product[NUM_PRODUCT_FIELDS] = {
"Product Manufacturer:",
"Product Name        :",
"Product Part Number :",
"Product Version     :",
"Product Serial Num  :",
"Product Asset Tag   :",
"Product FRU File ID :",
"Product OEM Field   :" };
char *asset_hdr = 
"   Asset Tag Length :";

typedef struct {
	uchar len :6;
	uchar type:2;
	} TYPE_LEN;

void
free_fru(void)
{
   if (frubuf != NULL) free(frubuf);
   return;
}

int
load_fru(void)
{
   int ret;
   uchar indata[16];
   uchar resp[18];
   int sresp;
   uchar cc;
   int sz;
   char fwords;
   ushort *fruoff;
   int i;
   int chunk;

   indata[0] = 0x00;
   sresp = sizeof(resp);
   ret = ipmi_cmd(GET_FRU_INV_AREA,indata,1,resp,&sresp,&cc,fdebug);
   if (fdebug) printf("load_fru: inv ret = %d, cc = %x\n",ret,cc);
   if (ret != 0) return(ret);
   if (cc != 0) return(cc);

   sz = resp[0] + (resp[1] << 8);
   if (resp[4] & 0x01) { fwords = 1; sz = sz * 2; }
   else fwords = 0;

   frubuf = malloc(sz);
   if (frubuf == NULL) return(errno);
   sfru = sz;
      
   /* Loop on READ_FRU_DATA */
   fruoff = (ushort *)&indata[1];
   for (i = 0, chunk=16; i < sz; i+=chunk)
   {
	if (i+16 >= sz) chunk = sz - i;
	indata[0] = 0x00;  /* FRU Device ID */
	if (fwords) {
	   indata[3] = chunk / 2;
	   *fruoff = (i/2);
	} else {
	   indata[3] = chunk;
	   *fruoff = i;
	}
	sresp = sizeof(resp);
	ret = ipmi_cmd(READ_FRU_DATA,indata,4,resp,&sresp,&cc,fdebug);
	if (ret != 0) break;
	else if (cc != 0) { 
		if (fdebug) printf("read_fru[%d]: cc = %x\n",i,cc);
		if (i == 0) ret = cc; 
		break; 
	}
        memcpy(&frubuf[i],&resp[1],chunk);
   }
   ret = ipmi_cmd(GET_SYSTEM_GUID,indata,0,resp,&sresp,&cc,fdebug);
   if (fdebug) printf("system_guid: ret = %d, cc = %x\n",ret,cc);
   if (ret == 0 && cc == 0) {
	if (fdebug) {
	   printf("system guid (%d): ",sresp);
	   for (i=0; i<17; i++) printf("%02x ",resp[i]);
	   printf("\n");
	}
	memcpy(&guid,&resp,16);
   	guid[16] = 0;
   }

   return(ret);
}

static void decode_string(unsigned char type,
                          unsigned char language_code,
                          unsigned char *source,
                          char          *target,
                          int           size)
{
  unsigned char *s = &source[0];
  unsigned char *d = &target[0];
 
  if (size == 0 || size == 1) return;
  memset(target, 0, size);
  if (type == STRING_DATA_TYPE_BCD_PLUS)
    {
    while(size)
      {
      if ((*s <= (unsigned char) 0x09))
        *d++ = (unsigned char) 0x30 + (unsigned char) *s++;
      else
        {
        if (*s == (unsigned char) 0x0A)
          *d++ = (unsigned char) ' ';
        else if (*s == (unsigned char) 0x0B)
          *d++ = (unsigned char) '-';
        else if (*s == (unsigned char) 0x0C)
          *d++ = (unsigned char) '.';
        else if ((*s <= (unsigned char) 0x0D) && (*s <= (unsigned char) 0x0F))
          {
          *d++ = (unsigned char) '*';
          }
        s++;
        }
      size --;
      }
      target[size] = 0x0;
    }
  else if (type == STRING_DATA_TYPE_SIX_BIT_ASCII)
    {
    printf("Six bit ASCII decode not supported\n");
    }
  else if (type == STRING_DATA_TYPE_LANG_DEPENDANT)
    {
    if ((language_code == 0x00) || (language_code == 0x25) || (language_code == 0x19))
      {
      strncpy(target, source, size);
      target[size] = 0x0;
      }
    else
      {
      printf("Language 0x%x dependant decode not supported\n",
            language_code);
      return;
      }
    }
  else
    {
    printf("Unable to decode type 0x%.2x\n",type);
    return;
    }
}


uchar calc_cksum(uchar *pbuf,int len)
{
   int i; 
   uchar cksum;
   uchar sum = 0;
  
   for (i = 0; i < len; i++) sum += pbuf[i];
   cksum = 0 - sum; 
   return(cksum);
}

int
show_fru(void)
{
   int ret = 0;
   int i, sz;
   uchar *pfru;
   uchar lang;
   TYPE_LEN *tl;
   char newstr[64];

   /*
    * FRU header:
    *  0 = format_ver
    *  1 = internal_use offset
    *  2 = chassis_info offset
    *  3 = board_info offset   (usu 6 fields)
    *  4 = product_info offset (usu 8 fields)
    *  5 = multirecord offset
    *  6 = pad (00)
    *  7 = header checksum (zero checksum)
    */
   printf("Mainboard FRU Size  : %d\n",sfru);

   sz = 8;  /*minimum for common header*/
   for (i = 1; i < 6; i++) 	/* walk thru offsets */
	if (frubuf[i] != 0) sz = frubuf[i] * 8;
   if (sz > 8)   		/* if have at least one section */
      sz += frubuf[sz+1] * 8;   /* add length of last section */
   /* Now, sz = size used, sfru = total available size */
   if (sz > sfru) {
      printf("FRU size out of bounds: available=%d used=%d\n",sfru,sz);
      printf("Please apply the correct FRU/SDR diskette\n");
      return(-1);
   }

   if (fdebug) {
      int b1, b2, p1, p2, c1, c2;
      int i1, i2, j;
      uchar line[9];
      uchar a;
      printf("FRU Header: ");
      for (i = 0; i < 8; i++) printf("%02x ",frubuf[i]);
      printf("\n");
      printf("FRU Body (size=%d/%d): ",sz,sfru);
      line[8] = 0;  j = 0;
      for (i = 0; i < sfru; i++) {
	a = frubuf[i];
	if (a < 0x20 || a > 0x7f) a = '.';
	line[j++] = a;
	if (i % 8 == 0) { j = 0; printf("%s\n  %04d: ",line,i); }
	printf("%02x ",frubuf[i]);
      }
      printf("\n");
      i1 = frubuf[1] * 8;  i2 = frubuf[i1 + 1] * 8;
      c1 = frubuf[2] * 8;  c2 = frubuf[c1 + 1] * 8;
      b1 = frubuf[3] * 8;  b2 = frubuf[b1 + 1] * 8;
      p1 = frubuf[4] * 8;  p2 = frubuf[p1 + 1] * 8;
      printf("internal off=%d, len=%d, cksum = %02x\n",
		i1,i2,calc_cksum(&frubuf[i1],i2-1));
      printf("chassis off=%d, len=%d, cksum = %02x\n",
		c1,c2,calc_cksum(&frubuf[c1],c2-1));
      printf("board off=%d, len=%d, cksum = %02x\n",
		b1,b2,calc_cksum(&frubuf[b1],b2-1));
      printf("prod  off=%d, len=%d, cksum = %02x\n",
		p1,p2,calc_cksum(&frubuf[p1],p2-1));
      pfru = &frubuf[p1 + p2];
      tl = (TYPE_LEN *)pfru;
      for (i = 0; i < 10; i++)
      {
         if (pfru[0] == 0xC1) break;  /*end*/
         tl = (TYPE_LEN *)pfru;
         pfru++;
         {
	   newstr[0] = 0;
	   decode_string(tl->type,25,pfru,newstr,tl->len);
	   printf("OEM[%d] %s\n",i,newstr);
         }
         pfru += tl->len;
      }
   }  /*endif show header*/

   /* show chassis fields */
   pfru = &frubuf[(frubuf[2] * 8)];
   lang = 25;  /* English */
   ctype = pfru[2];  /*chassis type*/
   if (fdebug) printf("ctype=%x\n",ctype);
   if (ctype >= MAX_CTYPE) ctype = MAX_CTYPE - 1;
   printf("%s %s\n",ctype_hdr,ctypes[ctype]);
   pfru += 3;  /* skip board header */
   for (i = 0; i < NUM_CHASSIS_FIELDS; i++)
   {
      if (pfru[0] == 0xC1) break;  /*end*/
      tl = (TYPE_LEN *)pfru;
      pfru++;
      {
	newstr[0] = 0;
	decode_string(tl->type,lang,pfru,newstr,tl->len);
	printf("%s %s\n",chassis[i],newstr);
      }
      pfru += tl->len;
   }
   if (fdebug) printf("num Chassis fields = %d\n",i);

   /* show board fields */
   pfru = &frubuf[(frubuf[3] * 8)];
   lang = pfru[2];
   pfru += 6;  /* skip board header */
   for (i = 0; i < NUM_BOARD_FIELDS; i++)
   {
      if (pfru[0] == 0xC1) break;  /*end*/
      tl = (TYPE_LEN *)pfru;
      pfru++;
      {
	newstr[0] = 0;
	decode_string(tl->type,lang,pfru,newstr,tl->len);
	printf("%s %s\n",board[i],newstr);
      }
      pfru += tl->len;
   }
   if (fdebug) printf("num Board fields = %d\n",i);

   /* show product fields */
   pfru = &frubuf[(frubuf[4] * 8)];
   maxprod = pfru[1] * 8;
   lang = pfru[2];
   pfru += 3;  /* skip product header */
   for (i = 0; i < NUM_PRODUCT_FIELDS; i++)
   {
      if (*pfru == 0xC1) break;  /*end*/
      if (*pfru == 0) *pfru = 0xc0;   /* %%%% fix broken table */
      tl = (TYPE_LEN *)pfru;
      if (i == 5) {  /* asset tag */
	asset_offset = (pfru - frubuf);
	asset_len    = tl->len;
      }
      pfru++;
      {
	newstr[0] = 0;
	decode_string(tl->type,lang,pfru,newstr,tl->len);
	printf("%s %s\n",product[i],newstr);
      }
      pfru += tl->len;
   }
   if (fdebug) 
	printf("num Product fields = %d, last=%x, max = %d\n", 
		i,*pfru,maxprod );
   if (*pfru == 0x00) *pfru = 0xC1;  /* insert end char if broken */
   printf("System GUID         : ");
   for (i=0; i<16; i++) printf("%02x",guid[i]);
   printf("\n");
   printf("%s %d\n",asset_hdr,asset_len);
   return(ret);
}

#define WRITEFRU_SZ   16

int
write_asset(char *tag)
{
   int ret = -1;
   uchar newdata[256];  /* max 128 + 3 (usu 83) */
   uchar req[25];
   uchar resp[16];
   int sresp;
   uchar cc;
   ushort *fruoff;
   int len;
   TYPE_LEN *tl;
   uchar *pfru;
   uchar *pnew;
   int i, j, plen; 
   int prod_offset; 
   uchar chksum;

   /* 
    * Abort if offset is negative or it would clobber
    * fru header (8 bytes). 
    */
   len = strlen(tag);
   if (fdebug) 
	   printf("write_asset: asset_off=%d asset_len=%d len=%d sFRU=%d maxprod=%d\n",
		   asset_offset,asset_len,len,sfru,maxprod);
   if (asset_offset < 8) return -1;
   if (asset_offset + len > sfru) return -1;
   
   prod_offset = frubuf[4] * 8;
   plen = frubuf[prod_offset+1] * 8;      // length of product data 
   if (plen < asset_len) return -2;       // not enough room
   if (plen > sizeof(newdata)) return -2; // plen = sizeof(newdata);

   memset(newdata,0,plen);

   j = asset_offset - prod_offset;
   pfru = &frubuf[prod_offset]; 
   pnew = &newdata[0];
   memcpy(pnew,pfru,j);
   pfru += j; 
   pnew += j;
   tl = (TYPE_LEN *)pnew;
   tl->type = 0x03;
   tl->len  = len;
   memcpy(++pnew,tag,len);

   /* copy trailing fru data from saved copy */
   if (fdebug) printf("fru[%d]: %02x %02x %02x, j=%d, alen=%d\n",
		   (pfru-frubuf),pfru[0],pfru[1],pfru[2],j,asset_len);
   j += asset_len + 1;
   pfru += asset_len + 1;
   pnew += len;
   for (i = 0; i < (plen-j); i++) {
     pnew[i] = pfru[i];
     if (pfru[i] == 0xc1) break;
   }

   /* include new checksum (calc over Product area) */
   chksum = calc_cksum(&newdata[0],plen-1);
   newdata[plen-1] = chksum;

   if (fdebug) {
	printf("new buffer (%d):",plen);
        for (i = 0; i < plen; i++) {
		if (i % 8 == 0) printf("\n %04d: ",i);
		printf("%02x ",newdata[i]);
	}
	printf("\n");
   }

   /* Write the buffer in small 16-byte chunks */
   sresp = sizeof(resp);
   req[0] = 0x00;  /* FRU Device ID */
   fruoff = (ushort *)&req[1];
   *fruoff = prod_offset;
   for (i = 0; i < plen; i += WRITEFRU_SZ) {
	memcpy(&req[3],&newdata[i],WRITEFRU_SZ);
	if (fdebug) {
	   printf("write_fru_data[%d] (len=%d): ",i,WRITEFRU_SZ+3);
	   for (j = 0; j < WRITEFRU_SZ+3; j++) printf("%02x ",req[j]);
	   printf("\n");
	}
	ret = ipmi_cmd(WRITE_FRU_DATA,req,WRITEFRU_SZ+3,resp,&sresp,&cc,fdebug);
	if (cc != 0) ret = cc; 
	if (fdebug && ret == 0) 
		printf("write_fru_data[%d]: %d bytes written\n",i,resp[0]);
	if (ret != 0) break;
	*fruoff += WRITEFRU_SZ;
   }
   return(ret);
}

int
main(int argc, char **argv)
{
   int ret;
   char c;
   char DevRecord[16];

   printf("%s: version %s\n",progname,progver);
   while ( (c = getopt( argc, argv,"a:x?")) != EOF )
      switch(c) {
          case 'x': fdebug = 1;      break;
          case 'a':
		fasset = 1;
		if (optarg) asset_tag = (char *)strdup(optarg);
		break;
          default:
                printf("Usage: %s [-x] [-a asset_tag]\n", progname);
		printf("   -a  Sets the asset tag\n");
		printf("   -x  Display extra debug messages\n");
                exit(1);
      }               

   ret = ipmi_getdeviceid( DevRecord, sizeof(DevRecord));
   if (ret == 0) {
      uchar ipmi_maj, ipmi_min;
      ipmi_maj = DevRecord[4] & 0x0f;
      ipmi_min = DevRecord[4] >> 4;
      printf("-- BMC version %x.%x, IPMI version %d.%d \n",
             DevRecord[2],  DevRecord[3],
             ipmi_maj, ipmi_min);
      }
   else exit(1);


   /* load the FRU data */
   ret = load_fru();
   if (ret != 0) {
	printf("load_fru error %d\n",ret);
	exit(1);
	}

   /* display the FRU data */
   ret = show_fru();
   if (ret != 0) printf("show_fru error %d\n",ret);

   if (fasset) {
	printf("\nWriting new asset tag (%s) ...\n",asset_tag);
        ret = write_asset(asset_tag);
        if (ret != 0) printf("write_asset error %d\n",ret);
	free_fru();
        ret = load_fru();
	if (ret != 0) printf("load_fru error %d\n",ret);
        else ret = show_fru();
	free(asset_tag);
   }

   free_fru();
   exit(ret);
}

/* end fruconfig.c */
