/*---------------------------------------------------------------------------
 * Filename:   pefconfig.c
 *
 * Author:     Andrew.R.Cress@intel.com
 * Copyright (c) 2001-2003 Intel Corporation. 
 *
 * Abstract:
 * This tool sets up the custom Platform Event Filter for the panicSEL 
 * record.  (0x20, OS Critical Stop).
 * It also configures the BMC LAN parameters, which support PEF alerts.
 *
 * ----------- Change History -----------------------------------------------
 * 10/16/01 Andy Cress - created
 * 10/24/01 Andy Cress - mods for SetPefEntry(0c)
 * 11/02/01 Andy Cress - added option to disable a given PefEntry
 * 11/15/01 Andy Cress - added function to GetLanEntry
 * 11/19/01 Andy Cress - added function to SetLanEntry
 * 01/18/02 Andy Cress - added GetCfgData function
 * 01/31/02 Andy Cress - converted to use ipmi_cmd_ia
 * 02/06/02 Andy Cress - removed GetCfgData
 * 02/06/02 Andy Cress - added ipmi_cmd_va 
 * 02/08/02 Andy Cress - added GetChanAcc
 * 02/14/02 Andy Cress - added Get_IPMac_Addr()
 * 02/21/02 Andy Cress - added Alert IP/MAC logic to Get_IPMac_Addr()
 * 03/21/02 Andy Cress - do SetChanAcc(0x80) with every SetChanAcc(0x40)
 * 04/15/02 Andy Cress v1.2 - fix bug with user-specified Alert IP:
 *		            (ccode=c7 on SetLanEntry(16 & 18), also
 *		            added Get_Mac() for user-specified Alert IP.
 * 04/16/02 Andy Cress v1.3 added SetUser() to set password if specified,
 *                          also changed unset mac[0] to be 0xFF, not 0.
 * 05/09/02 Andy Cress v1.4 fixed bug 504 gwymac[0]=00 (mymac overwrote it),
 *                          also fixed alert scan to pick last trapsink
 * 05/31/02 Andy Cress v1.5 for gwy mac, changed arping -c 1 to -c 2,
 *                          also get com2sec community from snmpd.conf,
 *                          set dest type for no ack, no retry,
 *                          special handling to set preset PEF entries
 * 07/02/02 Andy Cress v1.6 added more Usage text
 * 07/08/02 Andy Cress v1.7 SetUserAccess length change 
 * 08/02/02 Andy Cress v1.8  moved common ipmi_cmd() code to ipmicmd.c
 * 08/27/02 Andy Cress v1.9 fixed 0xc7 on SETUSER_ACCESS with pefconfig -P "",
 *                          show message if alert dest not found
 * 09/10/02 Andy Cress v1.10 make channel nums into defines (16)
 * 09/17/02 Andy Cress v1.11 decode Dest Addr IP (19) in decimal,
 *                           fix enable user padding in SetUser,
 *                           display new community string when setting
 * 12/09/02 Andy Cress v1.12 fix error w -C & snmpd.conf conflict
 * 01/29/03 Andy Cress v1.13 added MV OpenIPMI support
 * 02/05/03 Andy Cress v1.14 show pef entry descriptions,
 *                           added EnablePef routine
 *                           show correct Alert dest mac if -A only
 * 04/04/03 Andy Cress v1.15 add eth interface option (-i) 
 * 05/13/03 Andy Cress v1.16 fix EnablePef if startup delay not supported
 * 06/19/03 Andy Cress v1.17 added errno.h (email from Travers Carter) 
 * 07/25/03 Andy Cress v1.18 add SerialOverLan configuration
 *                           mod to SetUser, added GetBmcEthDevice,
 *                           use 'arping -I' if eth1.
 * 08/18/03 Andy Cress v1.19 Don't abort if IPMI 1.0, just skip PEF,
 *                           SetLanEntry(10 & 11) for bmc grat arp,
 *                           SetLanEntry(2) to 0x17 for auth priv
 * 09/10/03 Andy Cress v1.20 Don't enable a PEF entry if it is empty,
 *                           added -L lan_ch parameter,
 *                           scan for lan_ch in GetBmcEthDevice
 * 09/22/03 Andy Cress v1.21 Add DHCP option (-D), from Jerry Yu.
 * 12/05/03 Andy Cress v1.22 Fix auth type enables for ServerConfig
 * 12/16/03 Andy Cress v1.23 Allow flexible auth types via authmask
 * 03/19/04 Andy Cress v1.24 Change default pefnum for mBMC to 10
 */
/*M*
 *---------------------------------------------------------------------------
Copyright (c) 2002, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

  a.. Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer. 
  b.. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution. 
  c.. Neither the name of Intel Corporation nor the names of its contributors 
      may be used to endorse or promote products derived from this software 
      without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *---------------------------------------------------------------------------
 *M*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>
#include <sys/socket.h>    /* for Get_IPMac_Addr() */
#include <sys/ioctl.h>     /* for Get_IPMac_Addr() */
#include <net/if.h>        /* for Get_IPMac_Addr() */
#include <errno.h>
#include "ipmicmd.h" 
 
extern int errno;
#define SELprintf          printf
#define RTF_UP          0x0001   /* route usable */

#define SOL_ENABLE_PARAM		0x01
#define SOL_AUTHENTICATION_PARAM 	0x02
#define SOL_ENABLE_FLAG			0x01
#define SOL_PRIVILEGE_LEVEL_USER	0x02
#define SOL_BAUD_RATE_PARAM		0x05
#define SOL_PREFERRED_BAUD_RATE		0x07

   /* Channels: 0=IPMB, 1=Serial/EMP, 6=LAN2, 7=LAN1 */
   /* for Torrey Pines, LAN channel = 1 instead, see lan_ch below */
#define LAN_CH   7
#define SER_CH   1
#define MAXCHAN  16
#define NUM_DEVICES_TO_CHECK		32   /*for GetBmcEthDevice()*/

typedef struct
{            /* See IPMI Table 15-2 */
        uchar              rec_id;
        uchar              fconfig;
        uchar              action;
        uchar              policy;
        uchar              severity;
        uchar              genid1;
        uchar              genid2;
        uchar              sensor_type;
        uchar              sensor_no;
        uchar              event_trigger;
        uchar              data1;
        uchar              mask1;
        uchar              res[9];
}       PEF_RECORD;

typedef struct
{            /* See IPMI Table 19-3 */
        uchar              data[30];
}       LAN_RECORD;

/*
 * Global variables 
 */
char * progver   = "1.24";
char * progname  = "pefconfig";
char   fdebug    = 0;
char   fIPMI10   = 0;       /* =1 if IPMI v1.0 or less */
char   freadonly = 0;
char   fdisable  = 0;
char   fsetif    = 0;
char   fgetser   = 0;
char   fsetlan   = 0;
char   fsetifn   = 0;	     /* =1 if user specified ifname */
char   fpassword = 0;	     /* =1 user-specified a password, so set it. */
char   pefnum    = 0x0c;     /* 11 pre-defined entries, adding 12th */
uchar  rgmyip[4]   = {0,0,0,0};
uchar  rggwyip[4]  = {0,0,0,0};
uchar  rgdestip[4] = {0,0,0,0};
uchar  rgsubnet[4] = {0,0,0,0};  /* default subnet, i.e. 255.255.255.0 */
uchar  rgmymac[6]   = {0xFF,0,0,0,0,0};
uchar  rggwymac[6]  = {0xFF,0,0,0,0,0};
uchar  rgdestmac[6] = {0xFF,0,0,0,0,0};
char   rgcommunity[19] = "public";  /* default community */
char   fsetcommunity = 0;    /* =1 if user-specified community */
char   passwordData[17] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; 
uchar  authmask      = 0;  /* usu 0x17, mBMC = 0x15 */
uchar  lan_access    = 0;  /* usu 0x04 */
uchar  fmBMC         = 0;  
char   ifname[16]    = "eth0";
int    lan_ch        = LAN_CH;
int    lan_dhcp      = 0;
#define MAXPEF 31	 	/* max pefnum = 30 (31 entries), was 17 */
#define MAX_PEFPARAMS  14	/* max pef params = 14 */
uchar peflen[MAX_PEFPARAMS] = {0,1,1,1,1,1,21,2,1,4,17,1,3,18}; /*for ShowPef*/
uchar pef_array[MAXPEF][21];    /* array of all PEF entries read, */
                                /* sizeof(PEF_RECORD) = 21  */

char **pefdesc;
char *pefdesc1[MAXPEF] = {    /* for Sahalee */
/* 0 0x00 */ "",
/* 1 0x01 */ "BIOS ECC Error",
/* 2 0x02 */ "BIOS POST Error",
/* 3 0x04 */ "Chassis Intrusion",
/* 4 0x05 */ "FRB Failure",
/* 5 0x08 */ "Fan Failure",
/* 6 0x0c */ "Fatal NMI",
/* 7 0x0f */ "Power Supply Fault",
/* 8 0x07 */ "System Restart",
/* 9 0x13 */ "Temperature Sensor",
/*10 0x23 */ "Voltage Sensor",
/*11 0x12 */ "Watchdog Timer Reset",
/*12 0x20 */ "OS Critical Stop",
/*13 0x00 */ "undefined",
/*14 0x00 */ "undefined",
/*15 0x00 */ "undefined",
/*16 0x00 */ "undefined",
/*17 */ "reserved",
/*18 */ "reserved",
/*19 */ "reserved",
/*20 */ "reserved",
/*21 */ "reserved",
/*22 */ "reserved",
/*23 */ "reserved",
/*24 */ "reserved",
/*25 */ "reserved",
/*26 */ "reserved",
/*27 */ "reserved",
/*28 */ "reserved",
/*29 */ "reserved",
/*30 */ "reserved" };

char *pefdesc2[MAXPEF] = {    /* for mBMC */
/* 0 */ "",
/* 1 */ "Voltage Sensor Assert",
/* 2 */ "Proc FRB Thermal",
/* 3 */ "Voltage Sensor Deassert",
/* 4 */ "Proc1 IERR",
/* 5 */ "Digital Sensor OK",
/* 6 */ "Chassis Identify",
/* 7 */ "NMI Button",
/* 8 */ "Clear CMOS via Panel",
/* 9 */ "OS Load POST Code",
/*10 */ "OS Critical Stop",
/*11 */ "reserved",
/*12 */ "reserved",
/*13 */ "reserved",
/*14 */ "reserved",
/*15 */ "reserved",
/*16 */ "reserved",
/*17 */ "reserved",
/*18 */ "reserved",
/*19 */ "reserved",
/*20 */ "reserved",
/*21 */ "reserved",
/*22 */ "reserved",
/*23 */ "reserved",
/*24 */ "reserved",
/*25 */ "reserved",
/*26 */ "Chassis Intrusion",
/*27 */ "POST Code Error",
/*28 */ "Voltage Failure",
/*29 */ "Fan Failure",
/*30 */ "Temperature Failure" };

#define NLAN  27
struct {
  int cmd;
  int sz;
  char desc[28];
} lanparams[NLAN] = {   /* see IPMI Table 19-4 */
 /*  0 */  0, 1, "Set in progress",
 /*  1 */  1, 1, "Auth type support",
 /*  2 */  2, 5, "Auth type enables",
 /*  3 */  3, 4, "IP address",
 /*  4 */  4, 1, "IP addr src",  /* (DHCP/Static) */
 /*  5 */  5, 6, "MAC addr",
 /*  6 */  6, 4, "Subnet mask",
 /*  7 */  7, 3, "IPv4 header",
 /*  8 */  8, 2, "Prim RMCP port ",
 /*  9 */  9, 2, "Sec RMCP port ",
 /* 10 */ 10, 1, "BMC grat ARP ",
 /* 11 */ 11, 1, "grat ARP interval",
 /* 12 */ 12, 4, "Def gateway IP",
 /* 13 */ 13, 6, "Def gateway MAC",
 /* 14 */ 14, 4, "Sec gateway IP",
 /* 15 */ 15, 6, "Sec gateway MAC",
 /* 16 */ 16,18, "Community string",
 /* 17 */ 17, 1, "Num dest",
 /* 18 */ 18, 5, "Dest type",
 /* 19 */ 19, 13, "Dest address",
 /* 20 */ 96, 28, "OEM Alert String",
 /* 21 */ 97,  1, "Alert Retry Algorithm",
 /* 22 */ 98,  3, "UTC Offset",
 /* 23 */ 192, 4, "DHCP Server IP",
 /* 24 */ 193, 6, "DHCP MAC Address",
 /* 25 */ 194, 1, "DHCP Enable",
 /* 26 */ 201, 2, "Channel Access Mode(Lan)"
};

#define NSER  22   /* max=32 */
struct {
  int cmd;
  int sz;
  char desc[28];
} serparams[NSER] = {   /* see IPMI Table 20-4 */
 /*  0 */  0, 1, "Set in progress",
 /*  1 */  1, 1, "Auth type support",
 /*  2 */  2, 5, "Auth type enables",
 /*  3 */  3, 1, "Connection Mode",
 /*  4 */  4, 1, "Sess Inactiv Timeout",
 /*  5 */  5, 5, "Channel Callback",
 /*  6 */  6, 1, "Session Termination",
 /*  7 */  7, 2, "IPMI Msg Comm",
 /*  8 */  8, 2, "Mux Switch",
 /*  9 */  9, 2, "Modem Ring Time",
 /* 10 */ 10,17, "Modem Init String",
 /* 11 */ 11, 5, "Modem Escape Seq",
 /* 12 */ 12, 8, "Modem Hangup Seq",
 /* 13 */ 13, 8, "Modem Dial Command",
 /* 14 */ 14, 1, "Page Blackout Interval",
 /* 15 */ 15,18, "Community String",
 /* 16 */ 16, 1, "Num of Alert Dest",
 /* 17 */ 17, 5, "Destination Info",
 /* 18 */ 18, 1, "Call Retry Interval",
 /* 19 */ 19, 3, "Destination Comm Settings",
 /* 20 */ 29, 2, "Terminal Mode Config",
 /* 21 */ 201, 2,"Channel Access Mode (Ser)"
};


int GetDeviceID(LAN_RECORD *pLanRecord)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	int status;
	uchar inputData[24];
	uchar completionCode;

	if (pLanRecord == NULL) return(-1);

        status = ipmi_cmd(GET_DEVICE_ID, inputData, 0, responseData,
                        &responseLength, &completionCode, fdebug); 

	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("GetDeviceID: completion code=%x\n", 
				completionCode); 
		} else {
			memcpy(pLanRecord,&responseData[0],responseLength);
			return(0);  // successful, done
		}
	}  /* endif */
	/* if get here, error */
 	return(-1);
}  /*end GetDeviceID() */

int GetChanAcc(uchar chan, uchar parm, LAN_RECORD *pLanRecord)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	int status;
	uchar inputData[24];
	uchar completionCode;

	if (pLanRecord == NULL) return(-1);
	responseLength = 3;
	inputData[0] = chan;
	inputData[1] = parm;  /* 0x80 = active, 0x40 = non-volatile */
	responseLength = sizeof(responseData);
        status = ipmi_cmd(GET_CHANNEL_ACC, inputData, 2, responseData,
                        &responseLength, &completionCode, fdebug); 

	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("GetChanAcc: completion code=%x\n", 
				completionCode); 
		} else {
			// dont copy first byte (Parameter revision, usu 0x11)
			memcpy(pLanRecord,&responseData[0],responseLength);
			return(0);  // successful, done
		}
	}  /* endif */
	/* if get here, error */
 	return(-1);
}  /*GetChanAcc()*/

int SetChanAcc(uchar chan, uchar parm, uchar val)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	int status;
	uchar inputData[24];
	uchar completionCode;

	if (fmBMC) return(0);  /* mBMC doesn't support this */
        /* parm: 0x80 = active, 0x40 = set non-vol*/
	responseLength = 1;
	inputData[0] = chan;  /* channel */
	inputData[1] = (parm & 0xc0) | (val & 0x3F); 
#ifdef OLD
	if (chan == LAN_CH) inputData[2] = lan_access;  /* LAN access, no chg */
	else                   /* serial defaults to 0x02 = User priv level */
#endif
	inputData[2] = (parm & 0xc0) | lan_access; /* set priv level to Admin */

	responseLength = sizeof(responseData);
        status = ipmi_cmd(SET_CHANNEL_ACC, inputData, 3, responseData,
                        &responseLength, &completionCode, fdebug); 

	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("SetChanAcc: completion code=%x\n", 
				completionCode); 
		} else {
			return(0);  // successful, done
		}
	}  /* endif */
	/* if get here, error */
 	return(-1);
}  /*SetChanAcc()*/

int SetUser(void)
{
      uchar responseData[MAX_BUFFER_SIZE];
      int responseLength = MAX_BUFFER_SIZE;
      int status, i;
      uchar completionCode;
      char  inputData[24];
      int ret = 0;
 
      if (fpassword) {  /* if the user specified a password, set it. */
         inputData[0] = 0x01;  /*user 1 = null user */
	 responseLength = sizeof(responseData);
         status = ipmi_cmd(GET_USER_NAME, inputData, 1, responseData, 
		        &responseLength, &completionCode, fdebug);
         SELprintf("GET_USERNAME: %x %x %x, status = %x, ccode=%x\n",
                responseData[0],responseData[1],responseData[2],
		status,completionCode);
	 if (fdebug) {
		char aname[17];
                printf("User1: ");
		for (i = 0; i < responseLength; i++) {
                   printf("%02x ",responseData[i]);
		   if (responseData[i] < 0x20) aname[i] = '.';
		   else aname[i] = responseData[i];
		}
		aname[16] = 0;
                printf(" %s\n",aname);
	 }

         inputData[0] = 0x01;  /*user 1 = null user */
         inputData[1] = 0x01;  /*enable user*/
	 responseLength = sizeof(responseData);
         status = ipmi_cmd(SET_USER_PASSWORD, inputData, 2, responseData, 
		        &responseLength, &completionCode, fdebug);
         SELprintf("SETUSERPSW - inputData = %x %x %x, status = %x, ccode=%x\n",
                inputData[0],inputData[1],inputData[2],
		status,completionCode);
	 if (status != 0) ret = completionCode;

         inputData[0] = 0x01;  /*user 1 = null user */
         inputData[1] = 0x02;  /*set password*/
         memset(&inputData[2],0,16);
         strcpy(&inputData[2],passwordData);
	 responseLength = sizeof(responseData);
	 if (fdebug) {
		char apsw[17];
		char c;
                printf("Pswd1: ");
		for (i = 0; i < 16; i++) {
		   c = inputData[i+2];
                   printf("%02x ",(unsigned char)c);
		   if (c < 0x20) apsw[i] = '.';
		   else apsw[i] = c;
		}
		apsw[16] = 0;
                printf(" %s\n",apsw);
	 }
         status = ipmi_cmd(SET_USER_PASSWORD, inputData, 18, responseData, 
		        &responseLength, &completionCode, fdebug);
         SELprintf("SETUSERPSW - inputData = %x %x %x, status = %x, ccode=%x\n",
                inputData[0],inputData[1],inputData[2],
		status,completionCode);
	 if (status != 0) ret = completionCode;

         inputData[0] = 0x01;  /*user 1 = null user */
         inputData[1] = 0x03;  /*test password*/
         memset(&inputData[2],0,16);
         strcpy(&inputData[2],passwordData);
	 responseLength = sizeof(responseData);
         status = ipmi_cmd(SET_USER_PASSWORD, inputData, 18, responseData, 
		        &responseLength, &completionCode, fdebug);
         SELprintf("TESTUSERPSW - inputData = %x %x %x, status = %x, ccode=%x\n",
                inputData[0],inputData[1],inputData[2],
		status,completionCode);

         inputData[0] = 0x90 | lan_ch;   /* = 0x97 */
         inputData[1] = 0x01;   /* user 1 */
         inputData[2] = lan_access;   /* admin */
         inputData[3] = 0x00;   /* User Session Limit, 0=not limited*/
	 responseLength = sizeof(responseData);
         status = ipmi_cmd(SET_USER_ACCESS, inputData, 4, responseData,
                        &responseLength, &completionCode, fdebug);
         SELprintf("SETUSER_ACCESS - inputData = %x %x %x, status = %x, ccode=%x\n",
		   (uchar)inputData[0],inputData[1],inputData[2],
		   status,completionCode);
	 if (status != 0) ret = completionCode;
      }  /*endif fpassword*/

      return(ret);
}  /*end SetUser()*/

int GetUser(void)
{
      uchar responseData[MAX_BUFFER_SIZE];
      int responseLength = MAX_BUFFER_SIZE;
      int status;
      uchar completionCode;
      char  inputData[24];

         inputData[0] = lan_ch;
         inputData[1] = 0x01;  /* user 1 */
	 responseLength = sizeof(responseData);
         status = ipmi_cmd(GET_USER_ACCESS, inputData, 2, responseData,
                        &responseLength, &completionCode, fdebug);
	 if (status == 0 && completionCode == 0)
            SELprintf("Get User Access(1): %02x %02x %02x %02x\n",
		   (uchar)responseData[0],responseData[1],responseData[2],
			responseData[3]);
	 else 
            SELprintf("Get User Access(1): status=%x, ccode=%x\n",
	       		status, completionCode);
	return(status);
}  /*end GetUser()*/

int GetSerEntry(uchar subfunc, LAN_RECORD *pLanRecord)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	uchar inputData[24];
	int status;
	uchar completionCode;
	uchar chan; uchar bset;

	if (pLanRecord == NULL)
	{
	   if (fdebug)
	       printf("GetSerEntry: error, output buffer is NULL\n");
 	   return (-1);
	}

        chan = SER_CH;  /* 1=EMP, 0=IPMB, 6=LAN2, 7=LAN1 */
	bset = 0;

	inputData[0]            = chan;  // flags, channel 3:0 (1=EMP)
	inputData[1]            = subfunc;  // Param selector 
	inputData[2]            = bset;  // Set selector 
	inputData[3]            = 0;  // Block selector
	if (subfunc == 10)  {
	   inputData[2] = 0;
	   inputData[3] = 1;
 	}

        status = ipmi_cmd(GET_SER_CONFIG, inputData, 4, responseData,
                        &responseLength, &completionCode, fdebug); 

	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("GetSerEntry: completion code=%x\n", 
				completionCode); // responseData[0]);
		} else {
			// dont copy first byte (Parameter revision, usu 0x11)
			memcpy(pLanRecord,&responseData[1],responseLength-1);
			//successful, done
			return(0);
		}
	}

	// we are here because completionCode is not COMPLETION_CODE_OK
	if (fdebug) 
		SELprintf("GetSerEntry: sendImbRequest completion code=%x\n",
                          completionCode);
 	return -1;
}

int GetLanEntry(uchar subfunc, LAN_RECORD *pLanRecord)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	uchar inputData[24];
	int status;
	uchar completionCode;
	uchar chan; uchar bset;

	if (pLanRecord == NULL)
	{
	   if (fdebug)
	       printf("GetLanEntry: error, output buffer is NULL\n");
 	   return (-1);
	}

        chan = lan_ch;  /* LAN 1 = 7 */
	if (subfunc == 18 || subfunc == 19) bset = 1;  /* dest id = 1 */
	else bset = 0;

	inputData[0]            = chan;  // flags, channel 3:0 (LAN 1)
	inputData[1]            = subfunc;  // Param selector (3 = ip addr)
	inputData[2]            = bset;  // Set selector 
	inputData[3]            = 0;  // Block selector

        status = ipmi_cmd(GET_LAN_CONFIG, inputData, 4, responseData,
                        &responseLength, &completionCode, fdebug); 

	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("GetLanEntry: completion code=%x\n", 
				completionCode); // responseData[0]);
		} else {
			// dont copy first byte (Parameter revision, usu 0x11)
			memcpy(pLanRecord,&responseData[1],responseLength-1);
			//successful, done
			return(0);
		}
	}

	// we are here because completionCode is not COMPLETION_CODE_OK
	if (fdebug) 
		SELprintf("GetLanEntry: sendImbRequest completion code=%x\n",
                          completionCode);
 	return -1;
}  /* end GetLanEntry() */

int SetLanEntry(uchar subfunc, LAN_RECORD *pLanRecord, int reqlen)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	uchar inputData[24];
	int status;
	uchar completionCode;

	if (pLanRecord == NULL)
	{
	   if (fdebug)
	       printf("SetLanEntry: error, input buffer is NULL\n");
 	   return (-1);
	}

	inputData[0]            = lan_ch;  // flags, channel 3:0 (LAN 1)
	inputData[1]            = subfunc;  // Param selector (3 = ip addr)
	memcpy(&inputData[2],pLanRecord,reqlen);

        status = ipmi_cmd(SET_LAN_CONFIG, inputData, reqlen+2, responseData,
                        &responseLength, &completionCode, fdebug); 

	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("SetLanEntry: completion code=%x\n", 
				completionCode); // responseData[0]);
		} else {
			//successful, done
			return(0);
		}
	}

	// we are here because completionCode is not COMPLETION_CODE_OK
	if (fdebug) 
		SELprintf("SetLanEntry: sendImbRequest completion code=%x\n",
                          completionCode);
 	return -1;
}  /* end SetLanEntry() */

int GetPefEntry(uchar subfunc, ushort rec_id, PEF_RECORD *pPefRecord)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	uchar inputData[24];
	int status;
	uchar completionCode;

	if (pPefRecord == NULL)
	{
	   if (fdebug)
	      printf("GetPefEntry(%d): error, output buffer is NULL\n",subfunc);
 	   return (-1);
	}

	inputData[0]            = subfunc; // Parameter = Evt Filter Table
	inputData[1]            = rec_id; 
	inputData[2]            = 0; 

        status = ipmi_cmd(GET_PEF_CONFIG, inputData, 3, responseData,
                        &responseLength, &completionCode, fdebug); 

        if (status == ACCESS_OK) {
                if( completionCode ) {
                        SELprintf("GetPefEntry(%d): completion code=%x\n",
                                subfunc,completionCode); // responseData[0]);
                } else {
                        // dont copy first byte (Parameter revision, usu 0x11)
                        memcpy(pPefRecord,&responseData[1],responseLength-1);
                        //successful, done
                        return(0);
                }
	}

	// we are here because completionCode is not COMPLETION_CODE_OK
	if (fdebug) 
		SELprintf("GetPefEntry: sendImbRequest completion code=%x\n",
                          completionCode);
 	return -1;
}  /* end GetPefEntry() */

int SetPefEntry(PEF_RECORD *pPefRecord)
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	uchar inputData[24]; /* sizeof(PEF_RECORD) = 21 +1=22 */
	int status;
	uchar completionCode;
	uchar subfunc;

	subfunc = 0x06; // Parameter = Evt Filter Table

	if (pPefRecord == NULL) {
	   if (fdebug)
	       printf("SetPefEntry: error, output buffer is NULL\n");
 	   return (-1);
	}

        //  0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21
        // 06 0c 80 01 01 00 ff ff 20 ff 6f ff 00 00 00 00 00 00 00 00 00 00
	// memset(&inputData[0],0,requestData.dataLength);
	inputData[0]            = subfunc; 
	memcpy(&inputData[1],pPefRecord,sizeof(PEF_RECORD));

        status = ipmi_cmd(SET_PEF_CONFIG, inputData, sizeof(PEF_RECORD)+1, 
		       responseData, &responseLength, &completionCode, fdebug); 

	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("SetPefEntry: completion code=%x\n", 
				completionCode); // responseData[0]);
		} else {
			//successful, done
			return(0);
		}

	}

	// we are here because completionCode is not COMPLETION_CODE_OK
	if (fdebug) 
		SELprintf("SetPefEntry: sendImbRequest completion code=%x\n",
                          completionCode);
 	return(-1);
 
}  /* end SetPefEntry() */

int DisablePef(void)
{
	uchar iData[24]; /* sizeof(PEF_RECORD) = 21 +1=22 */
	uchar rData[MAX_BUFFER_SIZE];
	int rLength = MAX_BUFFER_SIZE;
	uchar cc;
	int status;

        status = SetChanAcc(lan_ch, 0x80, 0x20);   /* PEF off, disabled */
        if (fdebug) SELprintf("SetChanAcc(lan/active), ret = %d\n",status);
        status = SetChanAcc(lan_ch, 0x40, 0x20);   /* PEF off, disabled */
        SELprintf("SetChanAcc(lan), ret = %d\n",status);
	if (status != 0) return(status);

	iData[0] = 0x01;        /* PEF Control Param */
	iData[1] = 0x00;	/* PEF disable */
        status = ipmi_cmd(SET_PEF_CONFIG, iData, 2, rData, &rLength, 
			  &cc, fdebug); 
	if (status != 0) return(status);
	if( cc ) {
		SELprintf("DisablePef[%d]: completion code=%x\n",iData[0],cc);
		return(-1);
	}
	return(status);
}

int ShowPef(void)
{
	uchar iData[24]; /* sizeof(PEF_RECORD) = 21 +1=22 */
	uchar rData[MAX_BUFFER_SIZE];
	int rLength = MAX_BUFFER_SIZE;
	uchar cc;
	int status, i,j;
	 
	for (j = 1; j < MAX_PEFPARAMS; j++) {
	   iData[0] = j;         /* PEF Control Param */
	   if (j == 6 || j == 7 || j == 9) iData[1] = 1; 
	   else iData[1] = 0x00;	 /* PEF Set Selector */
	   if (j == 13) iData[2] = 1; 
	   else iData[2] = 0x00;	 /* PEF Block Selector */
	   status = ipmi_cmd(GET_PEF_CONFIG, iData, 3, rData, &rLength, 
			  &cc, fdebug); 
	   if (status == 0 && cc == 0) {
		SELprintf("PefParam[%d]: ",iData[0]);
		for (i=0;i<peflen[j];i++) SELprintf("%02x ", rData[1+i]);
		SELprintf("\n");
	   } else
		SELprintf("PefParam[%d]: get_pef status=%d cc=%x\n",
			  	iData[0],status,cc);
	}
	return(status);
}

int EnablePef(void)
{
	uchar iData[24]; /* sizeof(PEF_RECORD) = 21 +1=22 */
	uchar rData[MAX_BUFFER_SIZE];
	int rLength = MAX_BUFFER_SIZE;
	uchar cc;
	int status;
	uchar sdelay;

        status = SetChanAcc(lan_ch, 0x80, 0x02);   /* PEF on, always avail */
        if (fdebug) SELprintf("SetChanAcc(lan/active), ret = %d\n",status);
        status = SetChanAcc(lan_ch, 0x40, 0x02);   /* PEF on, always avail */
        SELprintf("SetChanAcc(lan), ret = %d\n",status);
	if (status != 0) return(status);

	{
	  iData[0] = 0x01;         /* PEF Control Param */
	  iData[1] = 0x00;	   /* PEF Set Selector */
	  iData[2] = 0x00;	   /* PEF Block Selector */
          status = ipmi_cmd(GET_PEF_CONFIG, iData, 3, rData, &rLength, 
			  &cc, fdebug); 
	  if (status != 0 || cc != 0) sdelay = 0;
	  else sdelay = rData[1];
	  if (fdebug) SELprintf("EnablePef[%d]: get cc=%x, control=%02x\n",
			  	iData[0],cc,sdelay);
	  iData[0] = 0x01;         /* PEF Control Param (0x01 or 0x05) */
	  iData[1] = 0x01;	   /* PEF enable, & no startup delay */
          status = ipmi_cmd(SET_PEF_CONFIG, iData, 2, rData, &rLength, 
			  &cc, fdebug); 
	  if (status != 0) return(status);
	  if( cc ) {
		SELprintf("EnablePef[%d]: completion code=%x\n",iData[0],cc);
		return(-1);
	  }

#ifdef TEST
	  iData[0] = 0x01;         /* Serial Channel */
	  iData[1] = 0x13;         /* Dest Com settings = 19. */
	  iData[2] = 0x01;         /* POL Default Dest */
	  iData[3] = 0x60;         
	  iData[4] = 0x07;        
          status = ipmi_cmd(SET_SER_CONFIG, iData, 5, rData, &rLength, 
			  &cc, fdebug); 
#endif

	  iData[0] = 0x02;         /* PEF Action Param */
	  iData[1] = 0x00;	   /* PEF Set Selector */
	  iData[2] = 0x00;	   /* PEF Block Selector */
          status = ipmi_cmd(GET_PEF_CONFIG, iData, 3, rData, &rLength, 
			  &cc, fdebug); 
	  if (fdebug) SELprintf("EnablePef[%d]: get cc=%x, val=%02x\n",
			  	iData[0],cc,rData[1]);
	  iData[0] = 0x02;         /* PEF Action Param */
	  iData[1] = 0x2f;         /* enable alerts, reset, power cycle/down */
          status = ipmi_cmd(SET_PEF_CONFIG, iData, 2, rData, &rLength, 
			  &cc, fdebug); 
	  if (status != 0) return(status);
	  if( cc ) {
		SELprintf("EnablePef[%d]: completion code=%x\n",iData[0],cc);
		return(-1);
	  }

	  if ((sdelay & 0x04) != 0) {  /* startup delay is supported */
	    iData[0] = 0x03;         /* PEF Startup Delay Param */
	    iData[1] = 0x00;         /* 0 seconds, default is 0x3c (60 sec) */
            status = ipmi_cmd(SET_PEF_CONFIG, iData, 2, rData, &rLength, 
			  &cc, fdebug); 
	    if (status != 0) return(status);
	    if( cc ) {
		SELprintf("EnablePef[%d]: completion code=%x\n",iData[0],cc);
		// return(-1);
	    }
	    iData[0] = 0x04;         /* PEF Alert Startup Delay Param */
	    iData[1] = 0x00;         /* 0 seconds, default is 0x3c (60 sec) */
            status = ipmi_cmd(SET_PEF_CONFIG, iData, 2, rData, &rLength, 
			  &cc, fdebug); 
	    if (status != 0) return(status);
	    if( cc ) {
		SELprintf("EnablePef[%d]: completion code=%x\n",iData[0],cc);
		// return(-1);
	    }
	  } /*endif sdelay*/

	  iData[0] = 0x09;        /* PEF Alert Policy Table */
	  iData[1] = 0x01;        /* Policy number */
	  iData[2] = 0x18;        /* PEF LAN, always alert, policy enable */
	  iData[3] = 0x71;	/* LAN, default dest */
	  iData[4] = 0x00;        /* No alert string */
          status = ipmi_cmd(SET_PEF_CONFIG, iData, 5, rData, &rLength, 
			  &cc, fdebug); 
	  if (status != 0) return(status);
	  if( cc ) {
		SELprintf("EnablePef[%d]: completion code=%x\n",iData[0],cc);
		return(-1);
	  }
	}  /*endif IPMI 1.5 */

	return(status);
}  /* end EnablePef */

static int ReadSELinfo()
{
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	uchar completionCode;
	uchar inputData[6];
	int status;

        status = ipmi_cmd(GET_SEL_INFO, inputData, 0, responseData, 
			&responseLength, &completionCode, fdebug); 
        if (fdebug) 
		printf("ipmi_cmd(GET_SEL_INFO) status = %x, rlen=%d\n",
			status, responseLength);

	if (status == ACCESS_OK) {
                SELprintf("Code %d SEL Ver %d Support %d\n",
                          completionCode,
                          responseData[0],
                          responseData[13]
                          );
		//successful, done
		return(0);
	} else  return(1);

}  /*end ReadSELinfo()*/

uchar htoi(uchar *inhex)
{
   // char rghex[16] = "0123456789ABCDEF";
   uchar val;
   uchar c;
   c = inhex[0] & 0x5f;  /* force cap */
   if (c > '9') c += 9;  /* c >= 'A' */
   val = (c & 0x0f) << 4;
   c = inhex[1] & 0x5f;  /* force cap */
   if (c > '9') c += 9;  /* c >= 'A' */
   val += (c & 0x0f);
   return(val);
}

void atomac(uchar *array, char *instr)
{
   int i,j,n;
   char *pi;
   j = 0;
   pi = instr;
   n = strlen(instr);
   for (i = 0; i <= n; i++) {
      if (instr[i] == ':') {
	array[j++] = htoi(pi);
	pi = &instr[i+1];
	}
      else if (instr[i] == 0) {
	array[j++] = htoi(pi);
	}
   }
   if (fdebug) 
      printf("atomac: %02x %02x %02x %02x %02x %02x\n", 
         array[0],array[1],array[2],array[3], array[4],array[5]);
}  /*end atomac()*/

void atoip(uchar *array,char *instr)
{
   int i,j,n;
   char *pi;
   j = 0;
   pi = instr;
   n = strlen(instr);
   for (i = 0; i <= n; i++) {
      if (instr[i] == '.') {
	instr[i] = 0;
	array[j++] = atoi(pi);
	pi = &instr[i+1];
	}
      else if (instr[i] == 0) {
	array[j++] = atoi(pi);
	}
   }
   if (fdebug) 
      printf("atoip: %d %d %d %d\n", array[0],array[1],array[2],array[3]);
}  /*end atoip()*/

int GetBmcEthDevice(void)
{
    LAN_RECORD LanRecord;
    int devnum = -1;
    int ret;
    int nCurDevice;
    char szDeviceName[ 7 ];    /* MAX_DEVICE_NAME_LENGTH + 1 */
    uchar bmcMacAddress[ 6 ];  /*MAC_LENGTH*/
    int skfd;
    struct ifreq ifr;
    int rlen;
    uchar iData[2];
    uchar rData[10];
    uchar cc;
    int i = 0;
    int j;

    /* find the LAN channel via Channel Info */
    for (j = 1; j < MAXCHAN; j++) {
        iData[0] = j;  /*channel #*/
        rlen = sizeof(rData);
        ret = ipmi_cmd(GET_CHANNEL_INFO, iData, 1, rData, &rlen, &cc, 0); 
	if (ret != 0) break;
	if (rData[1] == 4) { /* LAN type*/
		if (fdebug) printf("chan[%d] = lan\n",j);
		lan_ch = j;
		i++;
	} else if (rData[1] == 5) { /* serial type*/
		if (fdebug) printf("chan[%d] = serial\n",j);
	} else  /* 7 = SMBus, 12 = System Interface */
		if (fdebug) printf("chan[%d] = %d\n",j,rData[1]);
    }
    if (fdebug) printf("lan_ch = %d\n",lan_ch);
    if (i == 0) return(-2);  /* no lan channels found */

	ret = GetLanEntry( 5 /*MAC_ADDRESS_LAN_PARAM*/, &LanRecord );
	if ( ret < 0 ) {
		printf( "GetBmcEthDevice: GetLanEntry failed\n" );
		return devnum;
	}

	if (fdebug) printf("BMC MAC %x:%x:%x:%x:%x:%x\n",
		LanRecord.data[0], LanRecord.data[1], LanRecord.data[2],
		LanRecord.data[3], LanRecord.data[4], LanRecord.data[5] );

	if ( ( skfd = socket(AF_INET, SOCK_DGRAM, 0 ) ) < 0) {
		if ( fdebug ) {
			perror("socket");
			return devnum;
		}
	}
	memcpy( bmcMacAddress, LanRecord.data, sizeof(bmcMacAddress) );

	for ( 	nCurDevice = 0 ;
		(nCurDevice < NUM_DEVICES_TO_CHECK) && (devnum == -1);
		nCurDevice++ )
	{
		sprintf( szDeviceName, "eth%d", nCurDevice );
		strcpy(ifr.ifr_name, szDeviceName );
		if (ioctl(skfd, SIOCGIFHWADDR, &ifr) > 0) {
			if ( fdebug ) 
				printf( "GetBmcEthDevice: Could not get MAC address for device %d\n", nCurDevice );
		} else {
			if (memcmp(ifr.ifr_hwaddr.sa_data, bmcMacAddress, sizeof(bmcMacAddress)) == 0) {
				devnum = nCurDevice;
				break;
			}
		}
	}

	if ( fdebug ) 
		printf( "GetBmcEthDevice: Returning Device %d\n", devnum );
	return devnum;
}

int Get_Mac(uchar *ipadr,uchar *macadr)
{
   FILE *fparp;
   char buff[1024]; 
   // char arpfile[] = "/proc/net/arp"; 
   char alertfile[] = "/tmp/dest.arping";
   char arping_cmd[128];
   char *pb, *pm, *px;
   int num, i;
   int foundit = 0;
   int ret = 0;

   /* Get a MAC address for a given IP address */
   if (ipadr[0] != 0) {   /* if valid IP address */

      /* make sure the destination is in the arp cache */
      sprintf(arping_cmd,
	      "arping -I %s -c 2 %d.%d.%d.%d |grep reply |tail -1 >%s\n",
              ifname,ipadr[0],ipadr[1],ipadr[2],ipadr[3],alertfile);
      if (fdebug) printf(arping_cmd);
      system(arping_cmd);

      fparp = fopen(alertfile,"r");
      if (fparp == NULL) {
          fprintf(stderr,"Get_Mac: Cannot open %s, errno = %d\n",
			alertfile,errno);
	  ret = -1;
       } else {
         while (fgets(buff, 1023, fparp)) {
	   /* should only run through loop once */
	   num = strcspn(buff," \t");    /* skip 1st word ("Unicast") */
	   i = strspn(&buff[num]," \t");
	   pb = &buff[num+i];
  	   if (strncmp(pb,"reply",5) == 0) {  /* valid output */
	      /* Find the ip address */
	      pb += 6 + 5;         /* skip "reply from " */
	      num = strcspn(pb," \t");
	      pb[num] = 0;
	      if (fdebug) printf("Alert ip=%s\n",pb);
	      // IP address should match input param 
 	      // if (rgdestip[0] == 0) atoip(rgdestip,pb);
	      /* Now find the mac address */
	      pm = strchr(&pb[num+1],'[');
	      if (pm == NULL) pm = &pb[num+2];  /* just in case */
	      pm++;
	      px = strchr(pm,']');
	      if (px == NULL) px = pm + 17;    /* just in case */
	      px[0] = 0;
	      if (fdebug) printf("Alert mac=%s\n",pm);
	      foundit = 1;
 	      if (macadr[0] == 0xFF) 
                  atomac(macadr,pm);
	      break;
	   }
         } /*end while*/
         fclose(fparp);
       }  /*end else file opened*/
   }  /*endif valid IP */
   else ret = -1;

   if (ret == -1 || foundit == 0) {  /* couldn't get it */
      if (rgdestmac[0] == 0xFF)  /* not specified by user */
         memcpy(macadr,rggwymac,6);  /* get to it from the default gateway */
      }
   return(ret);
}  /* end Get_Mac()*/

int Get_IPMac_Addr()
{
   char rtfile[] = "/proc/net/route";
   char arpfile[] = "/proc/net/arp";
   char snmpfile[] = "/etc/snmp/snmpd.conf";
   char alertfile[] = "/tmp/alert.arping";
   FILE *fprt;
   FILE *fparp;
   int fd, skfd; 
   struct ifreq ifr;
   uchar *pc;
   int rc = 0;
   int i,j;
   uchar bnetadr[4];
   uchar bgateadr[4];
   char gate_addr[128]; 
   char buff[1024]; 
   char alertname[60];
   int num;

   /* Get the IP address and associated MAC address specified. */
   /* Local for eth0; Default Gateway; Alert Destination */

   /* Create a channel to the NET kernel. */
   if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("socket");
        exit(1);
   }

   strcpy(ifr.ifr_name, ifname);
   if (ioctl(skfd, SIOCGIFHWADDR, &ifr) > 0)
	printf("ioctl(SIOCGIFHWADDR) error, errno = %d\n",errno);
   else {
	if (rgmymac[0] == 0xFF)   /* if not user-specified */
           memcpy(rgmymac, ifr.ifr_hwaddr.sa_data, 6);   
   }

   fd = skfd;   // get_socket_for_af(AF_INET);
   if (fd >= 0) {
        strcpy(ifr.ifr_name, ifname);
        ifr.ifr_addr.sa_family = AF_INET;
        if (ioctl(fd, SIOCGIFADDR, &ifr) < 0) {
	    printf("ioctl(SIOCGIFADDR) error, errno = %d\n",errno);
	    /* errno 99 here means that eth0 is not up/defined. */
	} else {
	    pc = &ifr.ifr_addr.sa_data[2];
	    if (fdebug)
	      printf("addr = %d.%d.%d.%d \n", pc[0],pc[1],pc[2],pc[3]);
 	    if (rgmyip[0] == 0)   /* if not user-specified */
               memcpy(rgmyip, pc, 4);   

            strcpy(ifr.ifr_name, ifname);
            if (ioctl(fd, SIOCGIFNETMASK, &ifr) < 0)
	        printf("ioctl(SIOCGIFNETMASK) error, errno = %d\n",errno);
            else {              // sizeof(struct sockaddr)
	       pc = &ifr.ifr_netmask.sa_data[2];
	       if (fdebug)
	          printf("subnet = %d.%d.%d.%d \n", pc[0],pc[1],pc[2],pc[3]);
 	       if (rgsubnet[0] == 0)   /* if not user-specified */
                  memcpy(rgsubnet, pc, 4); 
	    }
	}
	/* close(fd);  done, close the socket? */
   }

   /* Get the default gateway IP */
   /* cat /proc/net/route and save Gwy if Dest == 0 and Gateway != 0 */
   fprt = fopen(rtfile,"r");
   if (fprt == NULL) {
      fprintf(stderr,"route: Cannot open %s, errno = %d\n",rtfile,errno);
   } else {
      char rtfmt[]  = "%16s %128s %128s %X %d %d %d %128s %d %d %d\n";
      int  iflags, refcnt, use, metric, mss, window, irtt;
      char iface[16]; 
      char mask_addr[128], net_addr[128];
      ulong *pnet;
      ulong *pgate;

      pnet = (long *)&bnetadr[0];
      pgate = (long *)&bgateadr[0];
      while (fgets(buff, 1023, fprt)) {
        num = sscanf(buff, rtfmt,
                     iface, net_addr, gate_addr,
                     &iflags, &refcnt, &use, &metric, mask_addr,
                     &mss, &window, &irtt); 
        if (num < 10 || !(iflags & RTF_UP))
            continue;

	j = 6;
	for (i = 0; i < 4; i ++) {
            bnetadr[i] = htoi(&net_addr[j]);
            bgateadr[i] = htoi(&gate_addr[j]);
	    j -= 2;
	    }
        if ((*pnet == 0) && (*pgate != 0)) {  /* found default gateway */
	    if (fdebug)
	       printf("default gateway: %s, %d.%d.%d.%d\n",gate_addr,
                   bgateadr[0], bgateadr[1], bgateadr[2], bgateadr[3]);
 	    if (rggwyip[0] == 0)   /* if not user-specified */
	       memcpy(rggwyip,bgateadr,4);
	}
      } /*end while*/
      fclose(fprt);
   }  /*end-else good open */

   /* Get the default gateway MAC */
   /* scan the arp cache for a match with gateway */
   if (rggwyip[0] != 0) {
    char arping_cmd[128];
    sprintf(gate_addr,"%d.%d.%d.%d",
              rggwyip[0], rggwyip[1], rggwyip[2], rggwyip[3]);
    /* make sure the gateway is in the arp cache */
    sprintf(arping_cmd,"arping -I %s -c 2 %s >/dev/null \n",ifname,gate_addr);
    if (fdebug) printf(arping_cmd);
    system(arping_cmd);

    fparp = fopen(arpfile,"r");
    if (fparp == NULL) {
      fprintf(stderr,"arp: Cannot open %s, errno = %d\n",arpfile,errno);
    } else {
      char arpfmt[] = "%128s %16s %16s %128s %16s %16s\n";
      char iface[16], hw_type[16], rgflags[16], rgmask[16];
      char hw_addr[128], net_addr[128];

      while (fgets(buff, 1023, fparp)) {
        num = sscanf(buff, arpfmt,
                     net_addr, hw_type, rgflags, hw_addr,
		     rgmask,iface);
        if (num < 6) continue;
	if (fdebug)
	   printf("arp: %s %s\n",net_addr,hw_addr);
	if (strcmp(gate_addr,net_addr) == 0) {  /* found it */
	   if (fdebug)
	      printf("gateway mac: %s\n",hw_addr);
 	   if (rggwymac[0] == 0xFF)   /* if not user-specified */
	      atomac(rggwymac,hw_addr);
	}
      }
      fclose(fparp);
    }
   }  /*endif have a gateway ip */

   /* Get the Alert Destination IP */
   /* By default, attempt to obtain this from /etc/snmp/snmpd.conf. */
   /* cat /etc/snmp/snmpd.conf | grep trapsink |tail -1 | cut -f2 -d' ' */
   alertname[0] = 0;  /* default to null string */
   fprt = fopen(snmpfile,"r");
   if (fprt == NULL) {
      fprintf(stderr,"snmp: Cannot open %s, errno = %d\n",snmpfile,errno);
   } else {
      // char snmpfmt[] = "%20s %60s\n";
      // char *keywd, *value;
      while (fgets(buff, 1023, fprt)) {
	/* parse each line */
	/* skip leading whitespace here */
	j = strspn(&buff[0]," \t");
	if (strncmp(&buff[j],"com2sec",7) == 0) {  /* found community line */
	   /* usu like this: "com2sec demouser default  public" */
	   i = j + 7;
	   for (j = 0; j < 3; j++) {
	     num = strspn(&buff[i]," \t");
	     i += num;
	     num = strcspn(&buff[i]," \t\r\n");
	     if (j < 2) i += num;
	   }
	   buff[i+num] = 0;
 	   if (fsetcommunity == 0)   /* if not user-specified */
	      strcpy(rgcommunity,&buff[i]);
	}
	if (strncmp(&buff[j],"trapsink",8) == 0) {  /* found it */
	   num = strspn(&buff[j+8]," \t");
	   i = j + 8 + num;
	   if (buff[i] == '`') continue;
	   num = strcspn(&buff[i]," \t\r\n");
	   buff[i+num] = 0;
	   strcpy(alertname,&buff[i]);
	   // break;  /* keep looking, use the last one */
	}
      } /*end while*/
      fclose(fprt);
      if (fdebug) 
	printf("snmp alertname=%s community=%s\n",alertname,rgcommunity);
   }  /*end else snmpfile*/
   
   /* Get the Alert Destination MAC from the alertname. */
   /* Check SSU's algorithm to make sure this is right. */
   if (alertname[0] != 0) {
      char arping_cmd[128];
      char *pb, *pm, *px;
      int num, i;

      if (strncmp(alertname,"localhost",9) == 0)
      {   /* localhost (self) is the SNMP alert destination */
	 if (rgdestip[0] == 0)    /* if not user-specified */
	    memcpy(rgdestip,rgmyip,4);
	 if (rgdestmac[0] == 0xFF)   /* if not user-specified */
	    memcpy(rgdestmac,rgmymac,6);
      } else {
       /* make sure the destination is in the arp cache */
       sprintf(arping_cmd,"arping -I %s -c 2 %s |grep reply |tail -1 >%s\n",
               ifname,alertname,alertfile);
       if (fdebug) printf(arping_cmd);
       system(arping_cmd);

       fparp = fopen(alertfile,"r");
       if (fparp == NULL) {
          fprintf(stderr,"alert: Cannot open %s, errno = %d\n",alertfile,errno);
       } else {
         while (fgets(buff, 1023, fparp)) {
	   /* should only run through loop once */
	   num = strcspn(buff," \t");    /* skip 1st word ("Unicast") */
	   i = strspn(&buff[num]," \t");
	   pb = &buff[num+i];
  	   if (strncmp(pb,"reply",5) == 0) {  /* valid output */
	      /* Find the ip address */
	      pb += 6 + 5;         /* skip "reply from " */
	      num = strcspn(pb," \t");
	      pb[num] = 0;
	      if (fdebug) printf("Alert ip=%s\n",pb);
 	      if (rgdestip[0] == 0) {  /* if ip not user-specified */
	         atoip(rgdestip,pb);
		 /* Now find the mac address */
		 pm = strchr(&pb[num+1],'[');
	         if (pm == NULL) pm = &pb[num+2];  /* just in case */
	         pm++;
	         px = strchr(pm,']');
	         if (px == NULL) px = pm + 17;    /* just in case */
	         px[0] = 0;
	         if (fdebug) printf("Alert mac=%s\n",pm);
 	         if (rgdestmac[0] == 0xFF)   /* if not user-specified */
	            atomac(rgdestmac,pm);
	      }
	      break;
	   }
         } /*end while*/
         fclose(fparp);
       }  /*end else file opened*/
      } /*endif not local */
   }  /*endif have alertname*/

   return(rc);
} /* end Get_IPMac_Addr */

int ShowChanAcc(uchar bchan)
{
    LAN_RECORD LanRecord;
    int ret;
    uchar access;
    char *pstr;
    uchar *pb;
    int bret;

    bret = 0;
    if (bchan == lan_ch) pstr = "lan";
    else if (bchan == SER_CH) pstr = "ser";
    else pstr = "";
    ret = GetChanAcc(bchan, 0x40, &LanRecord);
    if (ret == 0) {
	pb = (uchar *)&bret;
	pb[0] = LanRecord.data[0];
	pb[1] = LanRecord.data[1];
    }
    access = LanRecord.data[0];
    printf("GetChanAcc(%s), ret = %d, new value = %02x\n",
		pstr,ret,access);
    switch (access & 0x03) {
	     case 0: printf("  Access = Disabled, ");     break;
	     case 1: printf("  Access = Pre-Boot, ");     break;
	     case 2: printf("  Access = Always Avail, "); break;
	     case 3: printf("  Access = Shared, ");       break;
	     }
    if (access & 0x20) printf("PEF Alerts Disabled\n"); /*0*/
    else printf("PEF Alerts Enabled\n");   /*1*/
    return(bret);
}

int SetupSerialOverLan( void )
{
	uchar requestData[24];
	uchar responseData[MAX_BUFFER_SIZE];
	int responseLength = MAX_BUFFER_SIZE;
	int status;
	uchar completionCode;

	memset(requestData, 0, sizeof(requestData));  /* zero-fill */
	requestData[0] = 0x00;
	requestData[1] = SOL_ENABLE_PARAM;
	requestData[2] = SOL_ENABLE_FLAG;
        status = ipmi_cmd(SET_SOL_CONFIG, requestData,3,responseData,
                        &responseLength, &completionCode, fdebug); 
	if (status == ACCESS_OK) {
		switch( completionCode ) {
		   case 0x00: /* success */
			break;
		   case 0xC1: /* unsupported command */
			SELprintf("SetupSerialOverLan: SOL not available on this platform\n");
			return 0;
		   default: /* other error */
			SELprintf("SetupSerialOverLan: SOL_ENABLE_PARAM ccode=%x\n",
					completionCode);
			return -1;
			break;
		}
	} else {
		SELprintf( "SET_SOL_CONFIG, enable SOL failed\n" );
	 	return -1;
 	}

	requestData[0] = 0x00;
	requestData[1] = SOL_AUTHENTICATION_PARAM;
	requestData[2] = 0x00; // block
	requestData[3] = 0x00; // selector
        status = ipmi_cmd(GET_SOL_CONFIG, requestData,4,responseData,
                        &responseLength, &completionCode, fdebug); 
	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("SetupSerialOverLan: GET_SOL_AUTHENTICATION_PARAM code=%x\n",
				completionCode);

			return -1;
		}
	} else {
		SELprintf( "SET_SOL_CONFIG, get SOL authentication failed\n" );
	 	return -1;
 	}

	requestData[0] = 0x00;
	requestData[1] = SOL_AUTHENTICATION_PARAM;
	requestData[2] = SOL_PRIVILEGE_LEVEL_USER | ( responseData[1] & 0x80 );
	status = ipmi_cmd(SET_SOL_CONFIG, requestData,3,responseData,
			&responseLength, &completionCode, fdebug);
	if (status == ACCESS_OK) {
		if( completionCode ) {
			SELprintf("SetupSerialOverLan: SET_SOL_AUTHENTICATION_PARAM code=%x\n",
				completionCode);

			return -1;
		}
	} else {
		SELprintf( "SET_SOL_CONFIG, set SOL authentication failed\n" );
	 	return -1;
 	}

	requestData[0] = 0x00;
	requestData[1] = SOL_BAUD_RATE_PARAM;
	requestData[2] = SOL_PREFERRED_BAUD_RATE;
	status = ipmi_cmd(SET_SOL_CONFIG, requestData,3,responseData,
			&responseLength, &completionCode, fdebug);
	if (status == ACCESS_OK) {
	   if( completionCode ) {
		SELprintf("SetupSerialOverLan: SET SOL_PREFFERED_BAUD code=%x\n",
			completionCode);

		return -1;
	   }
	} else {
		SELprintf( "SET_SOL_CONFIG, set SOL BAUD failed\n" );
	 	return -1;
 	}
	return 0;
}  /* SetupSerialOverLan */

int
main(int argc, char **argv)
{
   int ret;
   PEF_RECORD PefRecord;
   LAN_RECORD LanRecord;
   int i, idx, j;
   char c;

   // progname = argv[0];
   printf("%s ver %s \n",progname,progver);

   while ( (c = getopt( argc, argv,"rsxldDn:I:M:S:G:H:A:B:C:P:i:L:?")) != EOF ) 
      switch(c) {
          case 'x': fdebug = 1;     break;
          case 'r': freadonly = 1;  break;
          case 'l': fsetlan = 1;    break; /* not used, opposite of freadonly */
          case 'd': fdisable = 1;   break;
          case 'D': lan_dhcp = 1;   break;
          case 's': fgetser = 1;    break;
          case 'I':      /* My IP Address (eth0) */
		atoip(rgmyip,optarg);
		break;
          case 'M':      /* My MAC Address */
		atomac(rgmymac,optarg);
		break;
          case 'S':      /* Subnet IP Address */
		atoip(rgsubnet,optarg);
		break;
          case 'G':      /* Gateway IP Address */
		atoip(rggwyip,optarg);
		break;
          case 'H':      /* Gateway MAC Address */
		atomac(rggwymac,optarg);
		break;
          case 'A':      /* Alert Dest IP Address */
		atoip(rgdestip,optarg);
		break;
          case 'B':      /* Alert Dest MAC Address */
		atomac(rgdestmac,optarg);
		break;
          case 'C':      /* Community String */
		if (strlen(optarg) > 18) optarg[18] = 0;
		fsetcommunity = 1;
		strcpy(rgcommunity,optarg);
		break;
	  case 'P':      /* Password */
		fpassword = 1;
		if (strlen(optarg) > 16) optarg[16] = 0;
		strcpy(passwordData,optarg);
                if (fdebug) printf("Password = %s\n",passwordData);
		break;
          case 'L': 
		lan_ch = atoi(optarg);
		if (lan_ch >= MAXCHAN) lan_ch = LAN_CH; /*set back to default*/
		break;
          case 'n':      /* number/index in PEF table to insert new entry */
		pefnum = atoi(optarg);
		if (pefnum >= MAXPEF) pefnum = MAXPEF - 1;
		break;
	  case 'i':      /* eth interface (ifname) */
          	fsetifn = 1;
		if (strlen(optarg) > 16) optarg[16] = 0;
		strcpy(ifname,optarg);
                if (fdebug) printf("ifname = %s\n",ifname);
		break;
	  default:
                printf("Usage: %s [-drsxD -n pefnum -i eth1 ]\n", progname);
                printf("         \t [-I ipadr -M macadr -S subnet]\n");
                printf("         \t [-G gwyip -H gwymac -P password ]\n");
                printf("         \t [-A alertip -B alertmac -C community ]\n");
                printf("         \t [-L lan_channel_num ]\n");
                printf("where -d  Disables BMC LAN & PEF\n");
                printf("      -r  Read-only BMC LAN & PEF settings\n");
                printf("      -s  Show some Serial settings also \n");
                printf("      -x  Show eXtra debug messages\n");
                printf("      -D  Use DHCP instead of static IP\n");
                exit(1);
      }

   ret = GetDeviceID( &LanRecord);
   if (ret == 0) {
      uchar ipmi_maj, ipmi_min;
      ipmi_maj = LanRecord.data[4] & 0x0f;
      ipmi_min = LanRecord.data[4] >> 4;
      printf("-- BMC version %x.%x, IPMI version %d.%d \n",
             LanRecord.data[2],  LanRecord.data[3],
             ipmi_maj, ipmi_min);
      if (ipmi_maj >= 1 && ipmi_min >= 5) fIPMI10 = 0;  /* >= IPMI 1.5 is ok */
      else {
         printf("This IPMI v%d.%d system does not support LAN PEF records.\n",
			 ipmi_maj,ipmi_min);
	 fIPMI10 = 1;
	 exit(1);    /* won't handle PEF, nor normal LAN config */
         }
      if ((LanRecord.data[6] == 0x22) && (LanRecord.data[7] == 0x03)) {
	 fmBMC = 1;  /*NSC mBMC*/
	 if (pefnum == 12) pefnum = 10;  /* change default pefnum to 0x0a */
	 pefdesc = &pefdesc2[0];
      } else {
	 fmBMC = 0;  /*Intel Sahalee BMC*/
	 pefdesc = &pefdesc1[0];
      }
   }

   i = GetBmcEthDevice();
   if (i == -2) {  /* no lan channels */
	printf("This system does not support BMC LAN channels.\n");
	exit(1);
   } else if (i < 0) i = 0; 
   if (fsetifn == 0) sprintf(ifname,"eth%d",i);

   ret = ReadSELinfo();
   if (ret == 0) {
    if (!fIPMI10) {
      printf("\n%s: GetPefEntry ...\n",progname);
      for (idx = 1; idx <= pefnum; idx++)
      {
         ret = GetPefEntry( 0x06, idx, &PefRecord);
         if (ret == 0) {    // Show the PEF record
            uchar * pc; int sz;
            pc = (uchar *)&PefRecord;
            sz = 21;        // sizeof(PEF_RECORD) = 21
            memcpy(pef_array[idx], &PefRecord, sz);
	    if (PefRecord.fconfig & 0x80) pc = "enabled";
	    else pc = "disabled";
	    i = PefRecord.rec_id;
            printf("PEFilter(%.2d): %02x %s event - %s\n",
		idx, PefRecord.sensor_type, pefdesc[i],pc);
	    if (fdebug) {  /* show raw PEFilter record */
		pc = &PefRecord.rec_id;
		printf("raw PEF(%.2d):  ",pc[0]);
		for (i = 0; i < sz; i++) printf("%02x ",pc[i]);
		printf("\n");
	    }
         } else 
            printf("GetPefEntry(%x), ret = %d\n",idx,ret);
      }
      if (fdebug) ShowPef();
      ret = GetPefEntry(0x01, 0,(PEF_RECORD *)&LanRecord);  
      if (ret == 0) printf("PEF Control: %02x\n",LanRecord.data[0]);
      ret = GetPefEntry(0x02, 0,(PEF_RECORD *)&LanRecord);
      if (ret == 0) printf("PEF Action: %02x\n",LanRecord.data[0]);
      ret = GetPefEntry(0x03, 0,(PEF_RECORD *)&LanRecord);
      if (ret == 0) printf("PEF Startup Delay: %d sec\n",LanRecord.data[0]);
      if (!fmBMC) {
		ret = GetPefEntry(0x04, 0,(PEF_RECORD *)&LanRecord);
		if (ret == 0) printf("PEF Alert Startup Delay: %d sec\n",
					LanRecord.data[0]);
		/* fmBMC gets cc=0x80 here */
      }
      ret = GetPefEntry(0x09, 1,(PEF_RECORD *)&LanRecord);
      if (ret == 0) printf("PEF Alert Policy: %02x %02x %02x %02x\n",
				LanRecord.data[0], LanRecord.data[1], 
				LanRecord.data[2], LanRecord.data[3]);

      if (!freadonly) {
       printf("\n%s: SetPefEntry(%d) ...\n",progname,pefnum);
       for (idx = 1; idx <= pefnum; idx++)
       {
	 // Set & Enable all PEF records
         memset(&PefRecord.rec_id,0,sizeof(PEF_RECORD));
         PefRecord.rec_id  = idx; /* next record, or user-specified */
	 if (idx < pefnum) {  /* pefnum defaults to 0x0c */
	    /* set config however it was previously */
            memcpy(&PefRecord.rec_id,pef_array[idx],sizeof(PEF_RECORD));
	 } else {  /* new OS Crit Stop entry */
            // Set PEF values for 0x20, OS Critical Stop event 
            PefRecord.genid1        = 0xff;
            PefRecord.genid2        = 0xff;
            PefRecord.sensor_type   = 0x20; /* OS Critical Stop */
            PefRecord.sensor_no     = 0xff;
            PefRecord.event_trigger = 0x6f;
            PefRecord.data1         = 0xff;
            PefRecord.mask1         = 0x00;
	 }
         if (fdisable) {
	    if (idx < pefnum) PefRecord.fconfig = 0x40; /* disabled, preset */
	    else PefRecord.fconfig  = 0x00;             /* disabled, software */
            PefRecord.action   = 0x00;
            PefRecord.policy   = 0x00;
	 } else {
            if (PefRecord.sensor_type != 0) { /* not an empty PEF entry */
              PefRecord.action   = 0x01;
              PefRecord.policy   = 0x01;
	      if (idx >= pefnum) 
	          PefRecord.fconfig  = 0x80;         /* enabled, software */
	      else {          /* special handling for presets, 1 thru 11 */
	          PefRecord.fconfig  = 0x80;    /* enabled, software */
                  ret = SetPefEntry(&PefRecord);
                  if (ret == 0 && fdebug) 
                     printf("SetPefEntry(%d/80) successful\n",PefRecord.rec_id);
		  PefRecord.fconfig  = 0xC0;     /* enabled, preset */
	      }
	    }  /*endif not empty*/
	 }
         {  // Show the new PEF record before setting it.
            uchar * pc; int sz;
            pc = (uchar *)&PefRecord;
            sz = 21;
            printf("PEFilter(%d): ",PefRecord.rec_id);
            for (i = 0; i < sz; i++) printf("%02x ",pc[i]);
                  printf("\n");
         } 
         ret = SetPefEntry(&PefRecord);
         if (ret == 0 && fdebug) 
             printf("SetPefEntry(%d) successful\n", PefRecord.rec_id); 
       }  /*end for*/
      }
     }  /*end if not fIPMI10*/

     printf("\n%s: GetLanEntry ...\n",progname);
     for (idx = 0; idx < NLAN; idx++)
     {
	 int ival;
         if (idx == 8 || idx == 9) continue;	  /* not implemented */
	 ival = lanparams[idx].cmd;
         if (ival >= 96 && ival <= 98) continue;  /* not implemented */
         if (fmBMC && (ival >= 192 && ival <= 194)) continue;  /* no DHCP */
         if (ival == 201) {
	     uchar *pb;
             // ret = GetChanAcc(lan_ch, 0x40, &LanRecord);
             j = ShowChanAcc(lan_ch);
	     pb = (uchar *)&j;
	     LanRecord.data[0] = pb[0];
	     LanRecord.data[1] = pb[1];
	     if (j == 0) ret = -1;
	     else { 
		ret = 0;
		// if (pb[1] >= 0x04) lan_access = 0x04;
		// else lan_access = pb[1];
		}
	 } else {
             ret = GetLanEntry(ival, &LanRecord);
	 }
         if (ret == 0) {    // Show the LAN record
            uchar * pc; int sz;
            pc = (uchar *)&LanRecord;
            sz = lanparams[idx].sz; 
            printf("Lan Param(%d) %s: ",ival,lanparams[idx].desc);
	    if (idx == 1) {
		authmask = pc[0]; /* auth type support mask */
		/* if (fmBMC) authmask is usually 0x15, else 0x14 */
		}
	    if (idx == 16) { printf("%s \n",pc);      /* string */
	    } else {  /* print results for others */
              for (i = 0; i < sz; i++) {
	  	if (idx == 3 || idx == 6 || idx == 12 || idx == 14 || idx == 23)
		   printf("%d ",pc[i]);      /* IP addresses in dec */
		else if (idx == 19)  {  /* dest addr */
		   if (i > 2 && i < 7) {
			char *sepstr;
		        if (i == 3) printf("[");
			if (i == 6) sepstr = "] ";
			else sepstr = " ";
		      printf("%d%s",pc[i],sepstr);  /* IP address in dec */
		      }
		   else printf("%02x ",pc[i]);  /* show mac/etc. in hex */
		   }
		else printf("%02x ",pc[i]);  /* show in hex */
		}  /*end for*/
              printf("\n");
	      /* if (idx == 4) { pc[0]: 01 = static, 02 = DHCP, 03 = BIOS } */
	    }
         } else  /* ret != 0 */
            printf("GetLanEntry(%d), ret = %d\n",ival,ret);
     }  /*end for*/
     if (fmBMC) lan_access = 0x04; 
     else lan_access = 0x04;
     GetUser();

     if (fgetser) {
       printf("\n%s: GetSerEntry ...\n",progname);
       for (idx = 0; idx < NSER; idx++) {
   	 int ival;
         // if (idx == 9) continue; /* not implemented */
	 ival = serparams[idx].cmd;
         if (ival == 201) {
             ret = GetChanAcc(SER_CH, 0x40, &LanRecord);
	 } else {
             ret = GetSerEntry(ival, &LanRecord);
	 }
         if (ret == 0) {    // Show the SER record
            uchar * pc; int sz;
            pc = (uchar *)&LanRecord;
            sz = serparams[idx].sz; 
            printf("Serial Param(%d) %s: ",ival,serparams[idx].desc);
            if (idx == 10) {  /* modem init string */
                pc[sz] = 0;
                printf("%02x %s\n",pc[0],&pc[1]);
                }
            else if ((idx >= 11 && idx <= 13) || idx == 15) {  /* strings */
                printf("%s\n",pc);
                }
            else {
              for (i = 0; i < sz; i++) {
		printf("%02x ",pc[i]);  /* show in hex */
		}
              printf("\n");
	    } /*end else*/
	 }
       }  /*end for*/
      }  /*endif fgetser*/

      if (!freadonly)  //  && fsetlan)
      {
         printf("\n%s: SetLanEntry ...\n",progname);
         if (fdisable) {
	     if (!fIPMI10) {
               ret = DisablePef();
               printf("DisablePef, ret = %d\n",ret);
	     }
	 } else {
             ret = SetChanAcc(lan_ch, 0x80, 0x22);   /* PEF off, always avail*/
             if (fdebug) printf("SetChanAcc(lan/active), ret = %d\n",ret);
             ret = SetChanAcc(lan_ch, 0x40, 0x22);   /* PEF off, always avail*/
             if (fdebug) printf("SetChanAcc(lan/nonvol), ret = %d\n",ret);
	     j = SetUser();
	     }
         printf("SetChanAcc(lan), ret = %d\n",ret);
	 j = ShowChanAcc(lan_ch);
	 if (authmask == 0) authmask = 0x17;
	 LanRecord.data[0] = (0x17 & authmask);
	 LanRecord.data[1] = (0x17 & authmask);
	 LanRecord.data[2] = (0x06 & authmask); 
	 LanRecord.data[3] = (0x06 & authmask);
	 LanRecord.data[4] = 0;
         ret = SetLanEntry(2, &LanRecord, 5);
	 printf("SetLanEntry(2), ret = %d\n",ret);
	 // LanRecord.data[0] = 0x01;  

         /* Get the values to use from Linux eth0, etc. */
         ret = Get_IPMac_Addr();
	 if (lan_dhcp) {  /* use DHCP */
	    LanRecord.data[0] = 0x03;  /* address source = BIOS using DHCP */
            ret = SetLanEntry(4, &LanRecord, 1);
            printf("SetLanEntry(4), ret = %d\n",ret);
	    /* DHCP also relates to lan params 192, 193, 194 */
	    /* SSU/BIOS set up DHCP Server IP in param 192. */
	 } else {    /* use static IP */
          printf("%s \t\tip=%d.%d.%d.%d mac=%02x:%02x:%02x:%02x:%02x:%02x\n",
		ifname, rgmyip[0], rgmyip[1], rgmyip[2], rgmyip[3], 
		rgmymac[0], rgmymac[1], rgmymac[2], rgmymac[3], 
		rgmymac[4], rgmymac[5]);
	  if (rgmyip[0] != 0) {
	    memcpy(&LanRecord,rgmyip,4);
            ret = SetLanEntry(3, &LanRecord, 4);
            printf("SetLanEntry(3), ret = %d\n",ret);
	    LanRecord.data[0] = 0x01;    /* static IP address source */
            ret = SetLanEntry(4, &LanRecord, 1);
            printf("SetLanEntry(4), ret = %d\n",ret);
	    memcpy(&LanRecord,rgmymac,6);
            ret = SetLanEntry(5, &LanRecord, 6);
            printf("SetLanEntry(5), ret = %d\n",ret);
	    memcpy(&LanRecord,rgsubnet,4);
            ret = SetLanEntry(6, &LanRecord, 4);
            printf("SetLanEntry(6), ret = %d\n",ret);
	    LanRecord.data[0] = 0x01;    /*grat arp*/
            ret = SetLanEntry(10, &LanRecord, 1);
	    printf("SetLanEntry(10), ret = %d\n",ret);
	    LanRecord.data[0] = 0x04;    /*grat arp interval*/
            ret = SetLanEntry(11, &LanRecord, 1);
	    printf("SetLanEntry(11), ret = %d\n",ret);
	  } else {      /* error, don't continue */
            printf("Missing IP Address, can't continue\n");
	    exit(1);
	  }
	  if (rggwyip[0] != 0) {
            printf("gateway \tip=%d.%d.%d.%d mac=%02x:%02x:%02x:%02x:%02x:%02x\n",
		rggwyip[0], rggwyip[1], rggwyip[2], rggwyip[3],
		rggwymac[0], rggwymac[1], rggwymac[2], rggwymac[3], 
		rggwymac[4], rggwymac[5]);
	    /* Set the Default Gateway IP & MAC */
	    memcpy(&LanRecord,rggwyip,4);
            ret = SetLanEntry(12, &LanRecord, 4);
            printf("SetLanEntry(12), ret = %d\n",ret);
	    memcpy(&LanRecord,rggwymac,6);
            ret = SetLanEntry(13, &LanRecord, 6);
	    printf("SetLanEntry(13), ret = %d\n",ret);
	  }
	 } /* end-else static IP */
	 ret = SetupSerialOverLan();
	 SELprintf("SetupSerialOverLan: ret = %d\n",ret);
	 if (rgdestip[0] == 0) {
            printf("alert dest \taddress not specified\n");
	 } else {   /* valid alert dest ip */
	    if (rgdestmac[0] == 0xFF) /* if MAC not set from defaults/user */
               ret = Get_Mac(rgdestip,rgdestmac);
            printf("alert dest \tip=%d.%d.%d.%d mac=%02x:%02x:%02x:%02x:%02x:%02x\n",
		rgdestip[0], rgdestip[1], rgdestip[2], rgdestip[3],
		rgdestmac[0], rgdestmac[1], rgdestmac[2], rgdestmac[3], 
		rgdestmac[4], rgdestmac[5]);
            printf("snmp community \t%s\n",rgcommunity);
	    /* Only need the SNMP community if there is an Alert Destination */
	    memset(&LanRecord.data[0], 0, 18);  /* make sure zero-filled */
	    strcpy(&LanRecord.data[0],rgcommunity);
            ret = SetLanEntry(16, &LanRecord, 18);  
            printf("SetLanEntry(16), ret = %d\n",ret);
	    /* Set Alert Destination Type */
	    LanRecord.data[0] = 0x01;     /* dest id = 1 */
	    LanRecord.data[1] = 0x00;     /* dest type = PET, no ack */
	    LanRecord.data[2] = 0x01;     /* ack timeout / retry interval */
	    LanRecord.data[3] = 0x00;     /* no retries */
	    // LanRecord.data[4] = 0x69; 
            ret = SetLanEntry(18, &LanRecord, 4);
            printf("SetLanEntry(18), ret = %d\n",ret);
	    /* Set the Alert Destination IP & MAC */
	    LanRecord.data[0] = 0x01;     /* dest id = 1 */
	    LanRecord.data[1] = 0x00; 
	    LanRecord.data[2] = 0x00; 
	    memcpy(&LanRecord.data[3],rgdestip,4);
	    memcpy(&LanRecord.data[7],rgdestmac,6);
            ret = SetLanEntry(19, &LanRecord, 13);
            printf("SetLanEntry(19), ret = %d\n", ret);

	    /* Now enable/disable PEF since we have an Alert destination. */
	    if (!fdisable && !fIPMI10) {
              ret = EnablePef();
              printf("EnablePef, ret = %d\n",ret);
	      j = ShowChanAcc(lan_ch);
	    }
	 } /*endif valid alert*/
      }  /*endif not readonly*/

   }  /* if ok */
   exit(ret);
}  /* end main()*/


/**********************************************
[ Sample PEF output to enable BMC LAN Alerts ]
pefconfig ver 1.1 starting ...
-- BMC version 0.38, IPMI version 1.5
Code 0 SEL Ver 81 Support 15
pefconfig: GetPefEntry ...
PEFilter(1): 01 40 00 00 00 ff ff 01 ff 01 95 0a 00 00 00 00 00 00 00 00 00
PEFilter(2): 02 40 00 00 00 ff ff 02 ff 01 95 0a 00 00 00 00 00 00 00 00 00
PEFilter(3): 03 40 00 00 00 ff ff 04 ff 01 95 0a 00 00 00 00 00 00 00 00 00
PEFilter(4): 04 40 00 00 00 ff ff 05 05 03 01 00 00 00 00 00 00 00 00 00 00
PEFilter(5): 05 40 00 00 00 ff ff 08 ff 6f 06 00 00 00 00 00 00 00 00 00 00
PEFilter(6): 06 40 00 00 00 ff ff 0c 08 6f 02 00 00 00 00 00 00 00 00 00 00
PEFilter(7): 07 40 00 00 00 ff ff 0f 06 6f 01 00 00 00 00 00 00 00 00 00 00
PEFilter(8): 08 40 00 00 00 ff ff 07 ff 6f 1c 00 00 00 00 00 00 00 00 00 00
PEFilter(9): 09 40 00 00 00 ff ff 13 ff 6f 3e 03 00 00 00 00 00 00 00 00 00
PEFilter(10): 0a 40 00 00 00 ff ff 23 03 6f 0e 00 00 00 00 00 00 00 00 00 00
PEFilter(11): 0b 40 00 00 00 ff ff 12 ff 6f 02 00 00 00 00 00 00 00 00 00 00
PEFilter(12): 0c 00 00 00 00 ff ff 00 ff ff ff 00 00 00 00 00 00 00 00 00 00
pefconfig: SetPefEntry ...
PEFilter(1): 01 c0 01 01 00 ff ff 01 ff 01 95 0a 00 00 00 00 00 00 00 00 00
PEFilter(2): 02 c0 01 01 00 ff ff 02 ff 01 95 0a 00 00 00 00 00 00 00 00 00
PEFilter(3): 03 c0 01 01 00 ff ff 04 ff 01 95 0a 00 00 00 00 00 00 00 00 00
PEFilter(4): 04 c0 01 01 00 ff ff 05 05 03 01 00 00 00 00 00 00 00 00 00 00
PEFilter(5): 05 c0 01 01 00 ff ff 08 ff 6f 06 00 00 00 00 00 00 00 00 00 00
PEFilter(6): 06 c0 01 01 00 ff ff 0c 08 6f 02 00 00 00 00 00 00 00 00 00 00
PEFilter(7): 07 c0 01 01 00 ff ff 0f 06 6f 01 00 00 00 00 00 00 00 00 00 00
PEFilter(8): 08 c0 01 01 00 ff ff 07 ff 6f 1c 00 00 00 00 00 00 00 00 00 00
PEFilter(9): 09 c0 01 01 00 ff ff 13 ff 6f 3e 03 00 00 00 00 00 00 00 00 00
PEFilter(10): 0a c0 01 01 00 ff ff 23 03 6f 0e 00 00 00 00 00 00 00 00 00 00
PEFilter(11): 0b c0 01 01 00 ff ff 12 ff 6f 02 00 00 00 00 00 00 00 00 00 00
PEFilter(12): 0c 80 01 01 00 ff ff 20 ff 6f ff 00 00 00 00 00 00 00 00 00 00
 **********************************************/

/* end pefconfig.c */
